""" Automatic selector of underlying engine for SEG-Y opening. """
#pylint: disable=redefined-builtin
from .segyio_loader import SegyioLoader
from .memmap_loader import MemmapLoader


def Loader(path, engine='memmap', endian='big', strict=False, ignore_geometry=True):
    """ !!. """
    loader_class = _select_loader_class(engine)
    return loader_class(path=path, endian=endian, strict=strict, ignore_geometry=ignore_geometry)
open = File = Loader


def _select_loader_class(engine):
    if isinstance(engine, type):
        return engine

    engine = engine.lower()
    if engine == 'segyio':
        return SegyioLoader
    if engine == 'memmap':
        return MemmapLoader
    raise ValueError(f'Unknown engine type `{engine}`!')



""" !!. """
from functools import partial
from concurrent.futures import Future, Executor


try:
    from batchflow import Notifier
except ImportError:
    try:
        from tqdm.auto import tqdm
        def Notifier(pbar, *args, **kwargs):
            """ Progress bar. """
            if pbar:
                return tqdm(*args, **kwargs)
            return lambda iterator: iterator
    except ImportError:
        class Notifier:
            """ Dummy notifier. """
            def __init__(self, *args, **kwargs):
                _ = args, kwargs

            def __call__(self, iterator, *args, **kwargs):
                _ = args, kwargs
                return iterator

            def __enter__(self):
                return self

            def __exit__(self, _, __, ___):
                return None

            def update(self, n=1):
                """ Do nothing on update. """
                _ = n
notifier = Notifier



class ForPoolExecutor(Executor):
    """ A sequential executor of tasks in a for loop.
    Inherits `Executor` interface, so can serve as a drop-in replacement for either
    `ThreadPoolExecutor` or `ProcessPoolExecutor` when threads or processes spawning is undesirable.
    """

    def __init__(self, *args, **kwargs):
        _ = args, kwargs
        self.task_queue = []

    def submit(self, fn, /, *args, **kwargs):
        """Schedule `fn` to be executed with given arguments."""
        future = Future()
        self.task_queue.append((future, partial(fn, *args, **kwargs)))
        return future

    def shutdown(self, wait=True):
        """Signal the executor to finish all scheduled tasks and free its resources."""
        _ = wait
        for future, fn in self.task_queue:
            future.set_result(fn())
        self.task_queue = None



""" Init file. """
from .segyio_loader import SegyioLoader, SafeSegyioLoader
from .memmap_loader import MemmapLoader
from .loader import Loader, File, open #pylint: disable=redefined-builtin

__version__ = '1.0.0'



""" segfast is a library for interacting with SEG-Y files at high speed. """
import re
from setuptools import setup, find_packages


with open('segfast/__init__.py', 'r') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)


with open('README.md', 'r') as f:
    long_description = f.read()


setup(
    name='segfast',
    packages=find_packages(),
    version=version,
    url='https://github.com/analysiscenter/segfast',
    license='Apache License 2.0',
    author='Sergey Tsimfer',
    author_email='sergeytsimfer@gmail.com',
    description='A library for interacting with SEG-Y seismic data',
    long_description=long_description,
    long_description_content_type="text/markdown",
    zip_safe=False,
    platforms='any',
    install_requires=[
        'dill>=0.3.1.1',
        'numpy>=1.20.0',
        'pandas>=1.0.0',
        'numba>=0.54.0',
        'segyio>=1.9.0',
    ],
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],

)